# aerospace_perforated_panel.rb
require 'sketchup'

module AerospacePerforatedPanel
  class Generator
    INCH_TO_MM = 25.4
    ASPECT_RATIO = 2.5
    PLUGIN_KEY = "AerospacePerforatedPanel_v1"

    def self.reset_defaults
      defaults = {
        width:         600,
        height:        1200,
        thickness:     20,
        hole_depth:    20,
        hole_diameter: 25,
        horizontal_spacing: 2.5,
        vertical_spacing:  2.5,
        margin:        70,
        hole_shape:    "圆形",
        arrangement_mode: "整齐排列"
      }

      defaults.each { |k,v| Sketchup.write_default(PLUGIN_KEY, k.to_s, v) }
      UI.messagebox("参数已重置为默认值")
    end

    def initialize
      @params = load_saved_parameters
      show_input_ui
    end

    private

    def parse_float(str)
      str.to_s.gsub(',', '.').to_f
    end

    def load_saved_parameters
      {
        width:         parse_float(Sketchup.read_default(PLUGIN_KEY, "width", 600)),
        height:        parse_float(Sketchup.read_default(PLUGIN_KEY, "height", 1200)),
        thickness:     parse_float(Sketchup.read_default(PLUGIN_KEY, "thickness", 20)),
        hole_depth:    parse_float(Sketchup.read_default(PLUGIN_KEY, "hole_depth", 20)),
        hole_diameter: parse_float(Sketchup.read_default(PLUGIN_KEY, "hole_diameter", 25)),
        horizontal_spacing: parse_float(Sketchup.read_default(PLUGIN_KEY, "horizontal_spacing", 2.5)),
        vertical_spacing:  parse_float(Sketchup.read_default(PLUGIN_KEY, "vertical_spacing", 2.5)),
        margin:        parse_float(Sketchup.read_default(PLUGIN_KEY, "margin", 70)),
        hole_shape:    Sketchup.read_default(PLUGIN_KEY, "hole_shape", "圆形"),
        arrangement_mode: Sketchup.read_default(PLUGIN_KEY, "arrangement_mode", "整齐排列")
      }
    end

    def save_parameters
      @params.each do |key, value|
        stored_value = value.is_a?(Numeric) ? value.to_f : value
        Sketchup.write_default(PLUGIN_KEY, key.to_s, stored_value)
      end
    end

    def show_input_ui
      prompts = [
        "板材宽度（mm）:", 
        "板材高度（mm）:", 
        "板材厚度（mm）:",
        "孔洞深度（mm）:", 
        "孔洞直径（mm）:", 
        "水平孔洞距离倍数:",
        "垂直孔洞距离倍数:", 
        "四周边距（mm）:", 
        "选择生成孔洞形状:", 
        "选择孔洞排列方式:"
      ]
      
      list = [nil]*8 + ["圆形|胶囊形|菱形|五边形|六边形", "整齐排列|错行排列"]
      display_values = @params.values.map { |v| v.is_a?(Float) && v == v.floor ? v.to_i : v }
      results = UI.inputbox(prompts, display_values, list, "洞洞板参数设置")
      return unless results

      @params = {
        width:         parse_float(results[0]),
        height:        parse_float(results[1]),
        thickness:     parse_float(results[2]),
        hole_depth:    parse_float(results[3]),
        hole_diameter: parse_float(results[4]),
        horizontal_spacing: parse_float(results[5]),
        vertical_spacing:  parse_float(results[6]),
        margin:        parse_float(results[7]),
        hole_shape:    results[8],
        arrangement_mode: results[9]
      }

      if validate_parameters
        save_parameters
        generate_panel
      else
        show_error("参数验证失败")
      end
    end

    def validate_parameters
      errors = []
      %i[width height thickness].each do |k|
        errors << "#{k}必须大于0" if @params[k] <= 0
      end
      errors << "孔深不能超过厚度" if @params[:hole_depth] > @params[:thickness]
      errors << "边距不足（至少#{(@params[:hole_diameter]/2.0).round(1)}mm）" if @params[:margin] < @params[:hole_diameter]/2.0
      
      errors << "水平孔距倍数需≥1.0" if @params[:horizontal_spacing] < 1.0
      errors << "垂直孔距倍数需≥1.0" if @params[:vertical_spacing] < 1.0
      
      errors << "无效孔洞形状" unless ["圆形","胶囊形","菱形","五边形","六边形"].include?(@params[:hole_shape])
      errors << "无效排列方式" unless ["整齐排列","错行排列"].include?(@params[:arrangement_mode])
      errors.empty? ? true : (show_error(errors.join("\n")); false)
    end

    def generate_panel
      model = Sketchup.active_model
      model.start_operation("生成洞洞板", true)
      
      panel_group = model.entities.add_group
      create_base_panel(panel_group)
      hole_layout = calculate_dynamic_layout
      create_holes(panel_group, hole_layout)

      model.commit_operation
      show_success_message(hole_layout)
    rescue => e
      model.abort_operation
      show_error("生成失败: #{e.message}\n#{e.backtrace.join("\n")}")
    end

    def create_base_panel(group)
      width = mm_to_inch(@params[:width])
      height = mm_to_inch(@params[:height])
      thickness = mm_to_inch(@params[:thickness])

      face = group.entities.add_face(
        [0, 0, 0], [width, 0, 0], [width, height, 0], [0, height, 0]
      )
      face.reverse! if face.normal.z < 0
      face.pushpull(thickness)
    end

    def calculate_dynamic_layout
      hole_radius = @params[:hole_diameter] / 2.0
      h_spacing = @params[:hole_diameter] * @params[:horizontal_spacing]
      v_spacing = @params[:hole_diameter] * @params[:vertical_spacing]

      max_cols = ((@params[:width] - 2 * @params[:margin]) / h_spacing).floor + 1
      max_rows = ((@params[:height] - 2 * @params[:margin]) / v_spacing).floor + 1

      used_width = (max_cols - 1) * h_spacing
      used_height = (max_rows - 1) * v_spacing

      start_x = (@params[:width] - used_width) / 2.0
      start_y = (@params[:height] - used_height) / 2.0

      start_x = [start_x, @params[:margin]].max
      start_y = [start_y, @params[:margin]].max

      { 
        cols: max_cols,
        rows: max_rows,
        start_x: start_x,
        start_y: start_y,
        h_spacing: h_spacing,
        v_spacing: v_spacing 
      }
    end

    def create_holes(group, layout)
      z_position = mm_to_inch(@params[:thickness])
      depth = mm_to_inch(@params[:hole_depth])

      layout[:rows].times do |row|
        current_cols = layout[:cols]
        x_offset = 0.0

        if @params[:arrangement_mode] == "错行排列"
          x_offset = (row % 2) * layout[:h_spacing] / 2.0
          current_cols -= 1 if row.odd?
        end

        current_cols.times do |col|
          x_mm = layout[:start_x] + col * layout[:h_spacing] + x_offset
          y_mm = layout[:start_y] + row * layout[:v_spacing]

          x = mm_to_inch(x_mm)
          y = mm_to_inch(y_mm)

          case @params[:hole_shape]
          when "圆形"
            create_circular_hole(group, x, y, z_position, depth)
          when "胶囊形"
            create_capsule_hole(group, x, y, z_position, depth)
          when "菱形"
            create_diamond_hole(group, x, y, z_position, depth)
          when "五边形"
            create_ngon_hole(group, x, y, z_position, depth, 5)
          when "六边形"
            create_ngon_hole(group, x, y, z_position, depth, 6)
          end
        end
      end
    end

    def create_circular_hole(group, x, y, z, depth)
      radius = mm_to_inch(@params[:hole_diameter]/2.0)
      circle = group.entities.add_circle([x, y, z], [0,0,1], radius, 24)
      face = group.entities.add_face(circle.map { |e| e.start.position })
      face.pushpull(-depth) if face
    end

    def create_capsule_hole(group, x, y, z, depth)
      width = mm_to_inch(@params[:hole_diameter])
      height = mm_to_inch(@params[:hole_diameter] * ASPECT_RATIO)
      radius = width / 2.0
      y_top = y + (height / 2.0 - radius)
      y_bottom = y - (height / 2.0 - radius)

      points = []
      num_segments = 12

      (0..num_segments).each do |i|
        angle = Math::PI - (Math::PI * i / num_segments)
        points << Geom::Point3d.new(
          x + radius * Math.cos(angle),
          y_top + radius * Math.sin(angle),
          z
        )
      end

      points << Geom::Point3d.new(x + radius, y_bottom, z)

      (0..num_segments).each do |i|
        angle = -Math::PI * i / num_segments
        points << Geom::Point3d.new(
          x + radius * Math.cos(angle),
          y_bottom + radius * Math.sin(angle),
          z
        )
      end

      points << Geom::Point3d.new(x - radius, y_top, z)

      curve = group.entities.add_curve(points)
      face = group.entities.add_face(curve.map { |e| e.start.position })
      face.pushpull(-depth) if face
    end

    def create_diamond_hole(group, x, y, z, depth)
      width = mm_to_inch(@params[:hole_diameter])
      height = mm_to_inch(@params[:hole_diameter] * ASPECT_RATIO)
      points = [
        Geom::Point3d.new(x - width/2, y, z),
        Geom::Point3d.new(x, y + height/2, z),
        Geom::Point3d.new(x + width/2, y, z),
        Geom::Point3d.new(x, y - height/2, z),
        Geom::Point3d.new(x - width/2, y, z)
      ]
      curve = group.entities.add_curve(points)
      face = group.entities.add_face(curve.map { |e| e.start.position })
      face.pushpull(-depth) if face
    end

    def create_ngon_hole(group, x, y, z, depth, sides)
      radius = mm_to_inch(@params[:hole_diameter]/2.0)
      polygon = group.entities.add_ngon([x, y, z], [0,0,1], radius, sides)
      face = group.entities.add_face(polygon.map { |e| e.start.position })
      face.pushpull(-depth) if face
    end

    def mm_to_inch(mm_value)
      mm_value.to_f / INCH_TO_MM
    end

    def show_success_message(layout)
      actual_margin_x = (@params[:width] - (layout[:cols] - 1) * layout[:h_spacing]) / 2.0
      actual_margin_y = (@params[:height] - (layout[:rows] - 1) * layout[:v_spacing]) / 2.0

      message = "成功在原点生成洞洞板！\n" +
                "孔洞阵列：#{layout[:cols]}x#{layout[:rows]}\n" +
                "水平孔距：#{layout[:h_spacing].round(1)}mm\n" +
                "垂直孔距：#{layout[:v_spacing].round(1)}mm\n" +
                "四周边距：#{actual_margin_x.round(1)}mm"

      UI.messagebox(message)
    end

    def show_error(message)
      UI.messagebox("错误：#{message}")
    end
  end
end

# ====================== 新增扩展注册代码 ======================
module AerospacePerforatedPanel
  # 定义扩展信息（必须放在模块内部）
  EXTENSION = SketchupExtension.new(
    "洞洞板生成器", 
    File.join(File.dirname(__FILE__), "aerospace_perforated_panel.rb")
  ).tap do |ext|
    ext.version     = "2.1.0"
    ext.creator     = "醇青"
    ext.description = "参数化洞洞板生成工具"
  end

  # 注册扩展（必须放在模块内部）
  Sketchup.register_extension(EXTENSION, true)
end

# ====================== 原有菜单/工具栏注册代码 ======================
if !file_loaded?(__FILE__)
  begin
    # 获取插件目录路径
    plugin_dir = File.dirname(__FILE__)
    
    # ======== 工具栏部分 ========
    toolbar = UI::Toolbar.new("洞洞板生成器")

    # 生成按钮
    generate_command = UI::Command.new("生成洞洞板") {
      AerospacePerforatedPanel::Generator.new
    }
    generate_command.tooltip = "生成洞洞板"
    generate_command.small_icon = File.join(plugin_dir, "icons", "aerospace_panel.png")
    generate_command.large_icon = File.join(plugin_dir, "icons", "aerospace_panel.png")
    toolbar.add_item(generate_command)

    # 重置按钮
    reset_command = UI::Command.new("重置参数") {
      AerospacePerforatedPanel::Generator.reset_defaults
    }
    reset_command.tooltip = "恢复默认参数"
    reset_command.small_icon = File.join(plugin_dir, "icons", "aerospace_panel_reset.png")
    reset_command.large_icon = File.join(plugin_dir, "icons", "aerospace_panel_reset.png")
    toolbar.add_item(reset_command)

    toolbar.show

    # ======== 菜单部分 ========
    plugins_menu = UI.menu("Plugins")
    plugins_menu.add_item("生成洞洞板") { AerospacePerforatedPanel::Generator.new }
    plugins_menu.add_item("重置洞洞板参数") { AerospacePerforatedPanel::Generator.reset_defaults }
  rescue
    begin
      # 中文菜单
      chinese_menu = UI.menu("插件")
      chinese_menu.add_item("生成洞洞板") { AerospacePerforatedPanel::Generator.new }
      chinese_menu.add_item("重置洞洞板参数") { AerospacePerforatedPanel::Generator.reset_defaults }
    rescue => e
      UI.messagebox("菜单初始化失败: #{e.message}")
    end
  ensure
    file_loaded(__FILE__)
  end
end